@@ -22,7 +22,7 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
-__version__ = "0.5.3"
+__version__ = "0.5.5"
 
 import sys
 
@@ -83,7 +83,9 @@ from qff.price.finance import (
     get_history_fundamentals,
     get_fundamentals,
     get_stock_reports,
-    get_fundamentals_continuously
+    get_fundamentals_continuously,
+    get_stock_forecast,
+    get_stock_express
 )
 
 from qff.price.fetch import (
@@ -159,6 +161,6 @@ from qff.helper.formula import (
 )
 
 from qff.helper.indicator import ind_ma, ind_macd, ind_atr, ind_kdj, ind_rsi, ind_boll
-from qff.helper.common import filter_st_stock, filter_paused_stock, filter_20pct_stock, select_zt_stock
+from qff.helper.common import filter_st_stock, filter_paused_stock, filter_20pct_stock, select_zt_stock, filter_bj_stock
 from qff.price.cache import get_current_data, UnitData
 from qff.frame.evaluation import strategy_eval
