@@ -31,7 +31,7 @@ from typing import Dict, Optional
 from datetime import datetime
 from bson.regex import Regex
 from qff.tools.mongo import DATABASE
-from qff.tools.date import util_time_stamp, get_pre_trade_day, is_trade_day, get_real_trade_date, util_date_valid
+from qff.tools.date import get_pre_trade_day, is_trade_day, get_real_trade_date, util_date_valid
 from qff.tools.utils import util_code_tolist
 from qff.tools.logs import log
 from qff.frame.context import context
@@ -491,19 +491,28 @@ def get_security_info(code, market='stock'):
     :type code: 字符串
     :param market: 用来过滤code的类型,目前支持的type仅有['stock', 'index', 'etf]
     :type market: 字符串
-    :return: 一个对象, 有如下属性:
-        1. name: 中文名称
-        2. start: 上市日期, 字符串类型
-        3. end: 退市日期（股票是最后一个交易日，不同于摘牌日期）, 如果没有退市则为2200-01-01
+    :return: 一个字典对象, 有如下key值:
 
-    示例：
+        1. code: 股票代码
+        2. name: 中文名称
+        3. start: 上市日期, 字符串类型
+        4. end: 退市日期（股票是最后一个交易日，不同于摘牌日期）, 如果没有退市则为2200-01-01
+
+    :example:
+
+    ::
         # 获取股票000001的上市时间
         start_date = get_security_info('000001').start
         print(start)
+
     """
     coll = DATABASE.get_collection(market + '_list')
     cursor = coll.find({'code': code}, {'_id': 0})
-    return cursor[0]
+    try:
+        rtn = cursor[0]
+    except IndexError:
+        rtn = {'code': code, 'name': '代码不存在', 'start': '', 'end': ''}
+    return rtn
 
 
 def get_stock_list(date=None):
@@ -581,7 +590,14 @@ def get_stock_name(code=None, date=None):
     else:
         coll = DATABASE.stock_list
     cursor = coll.find(filter, {"_id": 0, "code": 1, "name": 1})
-    return {item["code"]: item["name"] for item in cursor}
+    rtn = {item["code"]: item["name"] for item in cursor}
+
+    # 修复stock_list和stock_name两个结合数据来源不一致造成的bug
+    if code is not None and len(code) > len(rtn):
+        diff = {i: "-" for i in code if i not in rtn.keys()}
+        rtn = dict(**rtn, **diff)
+
+    return rtn
 
 
 def get_index_name(code=None):
@@ -674,7 +690,7 @@ def get_paused_stock(code=None, date=None):
         log.error("参数date输入不合法！")
         return None
 
-    filter['date_stamp'] = util_time_stamp(date)
+    filter['date'] = date
     filter['vol'] = {'$lt': 1}
 
     coll = DATABASE.stock_day
